function [Matbeta, vec_err_n, vec_df, ITC, vec_combr, vec_combd] = mpa_june20( y, X, mub, nu )

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% y         = response vector
% X         = full dictionary
% mub       = max. no. of iterations
% nu        = controls how greedy the algorithm is

%Output:
% Matbeta   = p X mub (estimated coefficients)
% vec_err_n = mub X 1 (RSS/n)
% vec_df    = mub X 1 (df)
% ITC       = has mub cols
% Significance of the rows:
% SC1; SC2; ESC1; ESC2; gMDL1; gMDL2; EgMDL1; EgMDL2; BIC; EBIC1; EBIC2; AICC;
% KIC; KICC; MMLU; ESC1d; ESC2d; EgMDL1d; EgMDL2d; EBIC1d;
% vec_combr  = mub X 1 (log of binomial coefficient)
% vec_combd  = mub X 1 (log of binomial coefficient computed with gamma function)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[n,p] = size(X);

Matbeta = zeros(p,mub);
vec_err_n = Inf*ones(mub,1);
vec_df = Inf*ones(mub,1);
vec_combr = zeros(mub,1);
vec_combd = zeros(mub,1);
ITC = Inf*ones(22,mub);

%Transform data
y = y-mean(y);
for jj=1:p
    X(:,jj) = X(:,jj) - mean ( X(:,jj) );
    X(:,jj) = sqrt(n)*( X(:,jj)/norm(X(:,jj)) );
end

c = (X'*y)/n;
D = (X'*X)/n;
noy = norm(y)^2/n;
beta = zeros(p,1);
alpha = zeros(p,1);
A = eye(n,n);
s = 0;
df = 0;

%%%%%%%%%%%%%%%%%%%
G = zeros(p,p);
cumul = zeros(p,1);
%%%%%%%%%%%%%%%%%%%

for j=1:mub,
    if (s>p-1),
        return;
    end
    if df>n-3,
        return;
    end

    a = c-D*beta;
    [~,sj] = max(abs(a));
    beta(sj) = beta(sj) + nu*a(sj);
    
    est_n = beta'*D*beta;
    err_n = noy + est_n - 2*beta'*c;
    if err_n<=0,
        return;
    end
    Matbeta(:,j) = beta; %save
    vec_err_n(j) = err_n; %save
    if alpha(sj)==0,
        alpha(sj)=1;
        s = sum(alpha);
        if s==1,
            combr = log(p);
        else
            combr = combr + log( (p-s+1)/s );
        end 
    end
    vec_combr(j) = combr;
    
    %%%%%%%%%%%%%%%%%%%%%%% 
    if n > p
        v = -nu*D(sj,:)*G;
        v = v + nu^2*D(sj,:).*cumul';
        G(sj,:) = G(sj,:) + v;
        cumul(sj) = cumul(sj)+1;
        df = df + nu - v*D(:,sj); 
        vec_df(j) = df; %save
    else
        P = X(:,sj)*X(:,sj)'/n;
        A = (eye(n,n) - nu*P)*A;  
        df = n-trace(A);
        vec_df(j) = df; %save
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%
    
    %Extended criteria (apply gammaln)
    combd = gammaln(p+1) -gammaln(df+1) -gammaln(p-df+1);
    vec_combd(j) = combd;
    
    vec_itc = comp_itc(combd,n, noy, est_n, err_n, df, nu, combr, p);
    ITC(:,j) = vec_itc; %save
end

end %function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function vec_itc = comp_itc(combd, n, noy, est_n, err_n, df, nu, comb, p)

%SC
term3 = log(df) -(n-1)*log(n-df);
SC1 = n*log(err_n) +df*log( (est_n/df)/(err_n/(n-df)) ) +term3;
diff_n = noy-err_n;
SC2 = n*log(err_n) +df*log( (diff_n/df)/(err_n/(n-df)) ) +term3;

%ESC
zeta1 = nu;
% zeta2 = df/j;
ESC1 = SC1 +2*zeta1*comb;
ESC2 = SC2 +2*zeta1*comb;

%gMDL
gMDL1 = (n/2)*log(noy) + (1/2)*log(n);
gMDL2 = gMDL1;
if (est_n/noy) >= (df/n),
    gMDL1 = (n/2)*log(err_n*n/(n-df)) +(df/2)*log( (est_n/df)/(err_n/(n-df)) ) +log(n);
end
if (diff_n/noy) >= (df/n),
    gMDL2 = (n/2)*log(err_n*n/(n-df)) +(df/2)*log( (diff_n/df)/(err_n/(n-df)) ) +log(n);
end

%EgMDL
EgMDL1 = gMDL1 +zeta1*comb;
EgMDL2 = gMDL2 +zeta1*comb;

%BIC
BIC = n*log(err_n) +df*log(n);

%EBIC1
EBIC1 = BIC +2*zeta1*comb;

%EBIC1
gam = 1; %gam should take values in [0,1]
EBIC2 = n*log(err_n) +df*log(n) +df*4*gam*log(p);

%AICC
AICC = log(err_n) +( 1+df/n )/( 1-(df+2)/n );

vec_itc = [SC1; SC2; ESC1; ESC2; gMDL1; gMDL2; EgMDL1; EgMDL2; BIC; EBIC1; EBIC2; AICC];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%  NEW CRITERIA %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%KIC
%Seghouane & Bekara 2004, Eq. (4)
KIC = n*log(err_n) + 3*(df+1);

%KICC
%Seghouane & Bekara 2004, Eq. (7)
KICC = n*log(err_n) + (2*(df+1)*n)/(n-df-2) - n*psi( (n-df)/2 );

%MMLu
%Giurcaneanu et al 2011, Page 1678
%Schmidt & Makalic 2009, Eq. (11)
te1 = ((n-df)/2)*log(2*pi);
te2 = ((n-df)/2)*( 1+ log(n*err_n/(n-df)) );
te3 = (df/2)*log(pi*n*noy);
te4 = -gammaln(1+(df/2));
te5 = (1/2)*log(df+1);
MMLU = te1 + te2 + te3 + te4 + te5;

%MMLg
%Giurcaneanu et al 2011, Page 1678
%Schmidt & Makalic 2009, Eq. (20)
MMLG1 = (n/2)*( 1+log((n*err_n)/(n-df+2)) ) + (1/2)*log(n-1) + (1/2); 
MMLG2 = MMLG1;
md = max(df-2,1);
if ( (df>0) &&  ( est_n/md > (err_n/(n-df+2)) ) )
    MMLG1 = ((n-df+2)/2)*( 1+log((n*err_n)/(n-df+2)) ) + ((df-2)/2)*log(n*est_n/md) + (1/2)*log((n-df)*df^2);
end
if ( (df>0) &&  ( diff_n/md > (err_n/(n-df+2)) ) )
    MMLG2 = ((n-df+2)/2)*( 1+log((n*err_n)/(n-df+2)) ) + ((df-2)/2)*log(n*diff_n/md) + (1/2)*log((n-df)*df^2);
end
    
%Extended criteria (apply gammaln)
ESC1d = SC1 +2*combd;
ESC2d = SC2 +2*combd;
EgMDL1d = gMDL1 +combd;
EgMDL2d = gMDL2 +combd;
EBIC1d = BIC +2*combd;

vec_itc_new = [KIC; KICC; MMLU; MMLG1; MMLG2; ESC1d; ESC2d; EgMDL1d; EgMDL2d; EBIC1d];

vec_itc = [vec_itc; vec_itc_new];

end %function

